//========================================================================
//
// Form.cc
//
// Copyright 1996-2003 Glyph & Cog, LLC
// Copyright 2006 Julien Rebetez
//
//========================================================================

#include <config.h>

#ifdef USE_GCC_PRAGMAS
#pragma implementation
#endif

#include <stddef.h>
#include <string.h>
#include "goo/gmem.h"
#include "goo/GooString.h"
#include "Error.h"
#include "Object.h"
#include "Array.h"
#include "Dict.h"
#include "Form.h"
#include "UGooString.h"
#include "XRef.h"


//========================================================================
// FormField
//========================================================================

FormField::FormField(XRef* xrefA, Object *aobj, unsigned id, Ref aref) {
  Object obj1, obj2, obj3i;
  double t;
  ref = aref;
  ID = id;
  ok = gFalse;
  xref = xrefA;
  aobj->copy(&obj);
  Dict *dict = obj.getDict();
  
  // get rectangle
  if (!dict->lookup("Rect", &obj1)->isArray()) {
    error(-1, "Annotation rectangle is wrong type");
    goto err2;
  }
  if (!obj1.arrayGet(0, &obj2)->isNum()) {
    error(-1, "Bad annotation rectangle");
    goto err1;
  }
  x1 = obj2.getNum();
  obj2.free();
  if (!obj1.arrayGet(1, &obj2)->isNum()) {
    error(-1, "Bad annotation rectangle");
    goto err1;
  }
  y1 = obj2.getNum();
  obj2.free();
  if (!obj1.arrayGet(2, &obj2)->isNum()) {
    error(-1, "Bad annotation rectangle");
    goto err1;
  }
  x2 = obj2.getNum();
  obj2.free();
  if (!obj1.arrayGet(3, &obj2)->isNum()) {
    error(-1, "Bad annotation rectangle");
    goto err1;
  }
  y2 = obj2.getNum();
  obj2.free();
  obj1.free();
  //swap coords if needed
  if (x1 > x2) {
    t = x1;
    x1 = x2;
    x2 = t;
  }
  if (y1 > y2) {
    t = y1;
    y1 = y2;
    y2 = t;
  }

  

  ok = gTrue;

  err1:
    obj2.free();  
  err2:
    obj1.free();
}

FormField::FormField(FormField *dest)
{
  x1 = dest->x1;
  y1 = dest->y1;
  x2 = dest->x2;
  y2 = dest->x2;

  ok = dest->ok;

  type = dest->type;
}

FormField::~FormField()
{
  obj.free();
}

//------------------------------------------------------------------------
// FormFieldButton
//------------------------------------------------------------------------
FormFieldButton::FormFieldButton(XRef *xrefA, Object *aobj, unsigned id, Ref ref) : FormField(xrefA, aobj, id, ref)
{
  type = fieldButton;
  Object obj1;
  Dict *dict = aobj->getDict();

  btype = fieldButtonCheck; //check box by default
  if (dict->lookup("Ff", &obj1)->isInt()) {
    int flags = obj1.getInt();
    if (flags & 0x10000) { //push button
      btype = fieldButtonPush;
    } else if (flags & 0x8000) {//radio button
      btype = fieldButtonRadio;
    }
  }
  obj1.free();
  if (dict->lookup("Kids", &obj1)->isArray()) { //we have a field of buttons
    size = obj1.getArray()->getLength();
    kids = new ButtonKid[size];
    if (btype == fieldButtonPush) {
      printf("%i kids of type Push Button\n", obj1.getArray()->getLength());
    } else if (btype == fieldButtonRadio) {
      printf("%i kids of type Radio Button\n", obj1.getArray()->getLength());
    }
  } else { //single button
    printf("single button\n");
    size = 1;
    kids = new ButtonKid[size];
    this->getRect(&kids[0].x1, &kids[0].y1, &kids[0].x2, &kids[0].y2);
    aobj->copy(&kids[0].obj);
    kids[0].ref = ref;
  }
  obj1.free();

  if (dict->lookup("Opt", &obj1)->isArray()) {
    printf("Opt entry found\n");
  }
  obj1.free();

  //dump AP dict
  //TODO:Browse the AP dictionnary to find the name of the state(/Yes, /No)
  if (dict->lookup("AP", &obj1)->isDict()) {
    Dict *dict = obj1.getDict();
    int length = dict->getLength();
    printf("AP length: %i\n", length);
    for(int i=0; i<length; i++) {
      Object obj2;
      printf("\t/%s\n", dict->getKey(i)->getCString());
      dict->getValNF(i, &obj2);
      if(obj2.isDict()) {
        Dict *dict2 = obj2.getDict();
        int l2 = dict2->getLength();
        for(int j=0; j<l2; j++) {
          Object obj3;
          dict2->getValNF(j, &obj3);
          printf("\t\t/%s", dict2->getKey(j)->getCString());
          printf(" %i %i\n", obj3.getRef().num, obj3.getRef().gen);
          obj3.free();
        }
      }
      obj2.free();
    }
  }
  obj1.free();
}

void FormFieldButton::setKidState (int index, GBool state)
{
  if(index < 0 || index > size) {
    error(-1, "FormFieldButton::setKidState, out of bound index");
    return;
  }
  ButtonKid *kid = &kids[index];
  char* value;
  if(state) value = "Yes";
  else value = "No";

  Object *obj1 = new Object();
  obj1->initName(value);
  printf("setKidState on %i, state %s", index, value);
  kid->obj.getDict()->set("V", obj1);
  obj1 = new Object();
  obj1->initName(value);
  //modify the Appearance State entry as well
  kid->obj.getDict()->set("AS", obj1);

  //notify the xref about the update
  xref->setModifiedObject(&kid->obj, kid->ref);
}

ButtonKid* FormFieldButton::getKid (int index)
{
  if (index < 0 || index >= size) 
    return NULL;
  return &kids[index];
}

FormFieldButton::~FormFieldButton()
{
  delete [] kids;
}

//------------------------------------------------------------------------
// FormFieldText
//------------------------------------------------------------------------
FormFieldText::FormFieldText(XRef *xrefA, Object *aobj, unsigned id, Ref ref) : FormField(xrefA, aobj, id, ref)
{
  type = fieldText;
  content = NULL;
  Dict *dict = aobj->getDict();
  Object obj1;

  if (dict->lookup("V", &obj1)->isString()) {
    if(obj1.getString()->getLength() == 2 && obj1.getString()->hasUnicodeMarker()) {
      printf("Form.cc:44, skipped empty unicode string\n");
      UGooString c(*obj1.getString());
      printf("ugoo: %s\n", c.getCString());
      content = c.getCString();
    } else {
      printf("Form.cc:46, V : %s\n", obj1.getString()->getCString());
      content = strdup(obj1.getString()->getCString());
    }
  }
  obj1.free();

}

FormFieldText::~FormFieldText()
{

}

void FormFieldText::setContentCopy(char* new_content)
{
  Object obj1;

  if(content) 
    delete [] content; 

  if (new_content == NULL)
    content = NULL;
  else {
    content = strdup(new_content); 

    GooString *cont = new GooString(content);
    Object *obj1 = new Object();
    obj1->initString(cont);
    //printf("%s, %s, %s\n", content, cont->getCString(), obj1->getString());
    obj.getDict()->set("V", obj1);
    //notify the xref about the update
    xref->setModifiedObject(&obj, ref);
  }
}


//------------------------------------------------------------------------
// FormFieldChoice
//------------------------------------------------------------------------
FormFieldChoice::FormFieldChoice(XRef *xrefA, Object *dict, unsigned id, Ref ref) : FormField(xrefA, dict, id, ref)
{
  type = fieldChoice;
}

FormFieldChoice::~FormFieldChoice()
{

}

//------------------------------------------------------------------------
// FormFieldSignature
//------------------------------------------------------------------------
FormFieldSignature::FormFieldSignature(XRef *xrefA, Object *dict, unsigned id, Ref ref): FormField(xrefA, dict, id, ref)
{
  type = fieldSignature;
}

FormFieldSignature::~FormFieldSignature()
{

}

//------------------------------------------------------------------------
// Form
//------------------------------------------------------------------------

Form::Form(XRef *xrefA, Object* annots, unsigned page)
{
  FormField* field;
  Object obj1, obj2;
  int size;
  int i;
  unsigned count = 0;

  fields = NULL;
  size = 0;
  numFields = 0;
  pageNum = page;

  if (annots->isArray()) {
    for (i = 0; i < annots->arrayGetLength(); ++i) {
      if (annots->arrayGet(i, &obj1)->isDict()) {
        if (obj1.dictLookup("Subtype", &obj2)->isName("Widget")) {
          unsigned id = ((unsigned)pageNum << 16) + count;
          obj2.free();
          annots->arrayGetNF(i, &obj2); //store the indirect ref for further update
          Ref objRef = obj2.getRef();
          obj2.free();
          //printf("num: %i\t gen:%i\n", objRef.num, objRef.gen);
          if(obj1.dictLookup("FT", &obj2)->isName("Btn")) {
            field = new FormFieldButton(xrefA, &obj1,id,objRef);
            count++;
          } else if (obj2.isName("Tx")) {
            field = new FormFieldText(xrefA, &obj1,id,objRef);
            count++;
          } else if (obj2.isName("Ch")) { 
            field = new FormFieldChoice(xrefA, &obj1,id,objRef);
            count++;
          } else if (obj2.isName("Sig")) {
            field = new FormFieldSignature(xrefA, &obj1,id,objRef);
            count++;
          } else {
            goto err1;
          }
          if (field->isOk()) {
            if (numFields >= size) {
              size += 16;
              fields = (FormField**)greallocn(fields,size,sizeof(FormField*));
            }
            fields[numFields++] = field;
          } else {
            delete field;
          }
          
        }
err1:
        obj2.free();
      }
      obj1.free();
    }
  }

}

Form::~Form() {
  int i;
  for(i = 0; i< numFields; ++i)
          delete fields[i];
  gfree(fields);
}

FormField *Form::find(double x, double y) const {
  int i;
  for (i = numFields -1; i>= 0; --i) {
    if (fields[i]->inRect(x, y)) {
      return fields[i];
    }
  }
  return NULL;
}

FormField* Form::getFieldById (unsigned int id) const {
  int i;
  for (i=0; i<numFields; ++i) {
    if (fields[i]->getID() == id) {
      return fields[i];
    }
  }
  return NULL;
}
GBool Form::onField(double x, double y) const {
  int i;
  for (i =0; i<numFields; ++i) {
    if (fields[i]->inRect(x,y))
            return gTrue;
  }
  return gFalse;
}

