//========================================================================
//
// Form.h
//
// Copyright 2006 Julien Rebetez
// Copyright 1996-2003 Glyph & Cog, LLC
//
//========================================================================

#ifndef FORM_H
#define FORM_H

#ifdef USE_GCC_PRAGMAS
#pragma interface
#endif

#include "Object.h"

class GooString;
class UGooString;
class Array;
class Dict;


//------------------------------------------------------------------------
// FormFieldType
//------------------------------------------------------------------------

enum FormFieldType {
  fieldButton,
  fieldText,
  fieldChoice,
  fieldSignature,
  //kid type
  fieldButtonCheck,
  fieldButtonPush,
  fieldButtonRadio
};

class FormField {
public:
  virtual ~FormField();

  FormField *copy() { return new FormField(this); }

  // Was the FormField created successfully ?
  GBool isOk() { return ok; }

  // Check if point is inside the field rectangle
  GBool inRect(double x, double y)
    { return x1 <= x && x <= x2 && y1 <= y && y <= y2; }

  // Get the field rectangle
  void getRect(double *xa1, double *ya1, double *xa2, double *ya2)
    { *xa1 = x1; *ya1 = y1; *xa2 = x2; *ya2 = y2; }

  // Accessors.
  FormFieldType getType() { return type; }
  unsigned getID () { return ID; }
  Object* getObj() { return &obj; }
  Ref getRef() { return ref; }

protected:
  // Build a FormField from the dict, only callable from child-classes
  FormField(XRef* xrefa, Object *aobj, unsigned id, Ref aref);
  FormFieldType type;           // field type
  GBool ok;           // set if created successfully
  Ref ref;
  Object obj;
  XRef *xref;



private:
  FormField() {}
  FormField(FormField *dest);
  /*
  Field ID is a 32bits integer, calculated as follow :
  the first 16 bits are the field number, relative to the page
  the last 16 bits are the page number
  [page number | field number]
  (encoding) id = (pageNum << 16) + fieldNum;
  (decoding) pageNum = id >> 16; fieldNum = (id << 16) >> 16;
  */
  unsigned ID; 

  double x1, y1;                // lower left corner
  double x2, y2;                // upper right corner
};


//------------------------------------------------------------------------
// FormFieldButton
//------------------------------------------------------------------------

struct ButtonKid
{
 double x1, y1, x2, y2;
 Object obj;
 Ref ref;
};

class FormFieldButton: public FormField {
public:
  FormFieldButton(XRef *xrefA, Object *dict, unsigned id, Ref ref);

  int getNumKids() { return size; }
  ButtonKid* getKid(int index);
  FormFieldType getButtonType () { return btype; }

  void setKidState(int index, GBool state);
  
  virtual ~FormFieldButton();
protected:
  FormFieldType btype;
  int size;
  ButtonKid *kids;
};

//------------------------------------------------------------------------
// FormFieldText
//------------------------------------------------------------------------

class FormFieldText: public FormField {
public:
  FormFieldText(XRef *xrefA, Object *dict, unsigned id, Ref ref);
  
  char* getContent() { return content; }
  char* getContentCopy() { if (content == NULL) return NULL;
                           else return strdup(content); }
  void setContentCopy(char* new_content);

  virtual ~FormFieldText();
protected:
  char* content;
};

//------------------------------------------------------------------------
// FormFieldChoice
//------------------------------------------------------------------------

class FormFieldChoice: public FormField {
public:
  FormFieldChoice(XRef *xrefA, Object *dict, unsigned id, Ref ref);

  virtual ~FormFieldChoice();
};

//------------------------------------------------------------------------
// FormFieldSignature
//------------------------------------------------------------------------

class FormFieldSignature: public FormField {
public:
  FormFieldSignature(XRef *xrefA, Object *dict, unsigned id, Ref ref);

  virtual ~FormFieldSignature();
};

//------------------------------------------------------------------------
// Form
//------------------------------------------------------------------------

class Form {
public:
  // Extract form from array of annotations
  Form(XRef *xrefA, Object* annots, unsigned page);

  ~Form();

  // Iterate through list of fields.
  int getNumFields() const { return numFields; }
  FormField* getField(int i) const { return fields[i]; }
  FormField* getFieldById (unsigned int id) const;

  // If point <x>,<y> is in a field, return the associated field;
  // else return NULL
  FormField *find(double x, double y) const;

  // Return true if <x>,<y> is in a field.
  GBool onField(double x, double y) const;

private:
  FormField** fields;
  int numFields;
  unsigned pageNum;
};
#endif

