//========================================================================
//
// pdftotext.cc
//
// Copyright 2006 Julien Rebetez
//
//========================================================================

#include "config.h"
#include <poppler-config.h>
#include <stdio.h>
#include <stdlib.h>
#include "goo/GooString.h"
#include "goo/gmem.h"
#include "GlobalParams.h"
#include "Object.h"
#include "Stream.h"
#include "Array.h"
#include "Dict.h"
#include "XRef.h"
#include "Catalog.h"
#include "Page.h"
#include "PDFDoc.h"
#include "TextOutputDev.h"
#include "CharTypes.h"
#include "UnicodeMap.h"
#include "Error.h"
#include "UGooString.h"

void modifyCatalogLang(PDFDoc *doc, GooString* lang);

int main(int argc, char **argv)
{
  PDFDoc *doc;
  GBool ok;
  GooString *inputFileName;
  GooString *newLang;
  GooString *outputs[3];
  
  // parse args
  if (argc < 3 || argc > 4) {
    fprintf(stderr, "pdf-modification-test\n");
    printf("usage: <PDF-input-file> <PDF-output-dir> <Lang>\n");
    return 0;
  }

  inputFileName = new GooString(argv[1]);
  newLang = new GooString(argv[3]);
  outputs[0] = new GooString(argv[2]); outputs[0]->append("/no_update.pdf");
  outputs[1] = new GooString(argv[2]); outputs[1]->append("/update_full_rewrite.pdf");
  outputs[2] = new GooString(argv[2]); outputs[2]->append("/update_incremental.pdf");
  doc = new PDFDoc(inputFileName, NULL, NULL);
  doc->saveAs(outputs[0], writeForceRewrite);
  modifyCatalogLang(doc, newLang);
  doc->saveAs(outputs[1], writeForceRewrite);
  doc->saveAs(outputs[2], writeForceIncremental);

  delete newLang;
  for(int i=0; i<3; i++) delete outputs[i];
  delete doc;
}

void modifyCatalogLang(PDFDoc* doc, GooString* lang)
{
  Object *catalog = new Object(); 
  XRef *xref = doc->getXRef();
  catalog = xref->getCatalog(catalog);

  Object o;
  catalog->dictLookup("Lang", &o);
  if(o.isString()) printf("input Lang: %s\n", o.getString()->getCString());
  if(catalog->isDict()) {
    Object* val = new Object();
    val->initString(lang);
    printf("output Lang: %s\n", val->getString()->getCString());
    catalog->dictSet("Lang", val);
  }
  Ref newRef;
  newRef.num = xref->getRootNum();
  newRef.gen = xref->getRootGen();
  xref->setModifiedObject(catalog, newRef);
}


